@@ -2,13 +2,62 @@ import os
 import datetime
 
 from celery import Celery, group
-from traitlets import Type, Unicode, Integer, List, default
+from traitlets import Type, Unicode, Integer, List, default, Callable, Bool
 from traitlets.config import LoggingConfigurable
 from sqlalchemy.pool import NullPool
+from conda.models.match_spec import MatchSpec
 
 from conda_store_server import orm, utils, storage, schema, api, conda
 
 
+def conda_store_validate_specification(
+    conda_store: "CondaStore", namespace: str, specification: schema.CondaSpecification
+) -> schema.CondaSpecification:
+
+    # ============== MODIFICATION =================
+    # CondaStore.conda_default_channels
+    if len(specification.channels) == 0:
+        specification.channels = conda_store.conda_default_channels.copy()
+
+    # CondaStore.conda_default_packages
+    if len(specification.dependencies) == 0:
+        specification.dependencies = conda_store.conda_default_packages.copy()
+
+    # CondaStore.conda_included_packages
+    dependency_names = set(
+        MatchSpec(_).name for _ in specification.dependencies if isinstance(_, str)
+    )
+    for package in set(conda_store.conda_included_packages) - dependency_names:
+        specification.dependencies.append(package)
+
+    # ================ VALIDATION ===============
+    normalized_conda_channels = set(
+        conda.normalize_channel_name(conda_store.conda_channel_alias, _)
+        for _ in specification.channels
+    )
+
+    # validate that all listed channels in specification are allowed
+    normalized_conda_allowed_channels = set(
+        conda.normalize_channel_name(conda_store.conda_channel_alias, _)
+        for _ in conda_store.conda_allowed_channels
+    )
+    if not (normalized_conda_channels <= normalized_conda_allowed_channels):
+        raise ValueError(
+            f"Conda channels {normalized_conda_channels - normalized_conda_allowed_channels} not allowed in specification"
+        )
+
+    # validate that required conda package are in specification
+    dependency_names = set(
+        MatchSpec(_).name for _ in specification.dependencies if isinstance(_, str)
+    )
+    if not (set(conda_store.conda_required_packages) <= dependency_names):
+        raise ValueError(
+            f"Conda packages {conda_store.conda_required_packages - dependency_names} required and missing from specification"
+        )
+
+    return specification
+
+
 class CondaStore(LoggingConfigurable):
     storage_class = Type(
         default_value=storage.S3Storage,
@@ -23,9 +72,15 @@ class CondaStore(LoggingConfigurable):
         config=True,
     )
 
+    build_directory = Unicode(
+        "{store_directory}/{namespace}",
+        help="Template used to form the directory for storing conda environment builds. Available keys: store_directory, namespace, name. The default will put all built environments in the same namespace within the same directory.",
+        config=True,
+    )
+
     environment_directory = Unicode(
-        "conda-store-state/envs",
-        help="directory for symlinking conda environment builds",
+        "{store_directory}/{namespace}/envs",
+        help="Template used to form the directory for symlinking conda environment builds. Available keys: store_directory, namespace, name. The default will put all environments in the same namespace within the same directory.",
         config=True,
     )
 
@@ -41,9 +96,42 @@ class CondaStore(LoggingConfigurable):
         config=True,
     )
 
+    conda_platforms = List(
+        [conda.conda_platform(), "noarch"],
+        help="Conda platforms to download package repodata.json from. By default includes current architecture and noarch",
+        config=True,
+    )
+
+    conda_default_channels = List(
+        ["conda-forge"],
+        help="Conda channels that by default are included if channels are empty",
+        config=True,
+    )
+
     conda_allowed_channels = List(
-        ["https://repo.anaconda.com/pkgs/main", "conda-forge"],
-        help="Allowed conda channels to be used in conda environments. Currently not enforced",
+        [
+            "main",
+            "conda-forge",
+        ],
+        help="Allowed conda channels to be used in conda environments",
+        config=True,
+    )
+
+    conda_default_packages = List(
+        [],
+        help="Conda packages that included by default if none are included",
+        config=True,
+    )
+
+    conda_required_packages = List(
+        [],
+        help="Conda packages that are required to be within environment specification. Will raise a validation error is package not in specification",
+        config=True,
+    )
+
+    conda_included_packages = List(
+        [],
+        help="Conda packages that auto included within environment specification. Will not raise a validation error if package not in specification and will be auto added",
         config=True,
     )
 
@@ -79,6 +167,12 @@ class CondaStore(LoggingConfigurable):
         config=True,
     )
 
+    serialize_builds = Bool(
+        True,
+        help="No longer build conda environment in parallel. This is due to an issue in conda/mamba that when downloading files in two concurent builds the downloads/extraction can overlap. This is a bug in conda/mamba that needs to be fixed.",
+        config=True,
+    )
+
     @default("celery_broker_url")
     def _default_celery_broker_url(self):
         return f"sqla+{self.database_url}"
@@ -120,6 +214,12 @@ class CondaStore(LoggingConfigurable):
         config=True,
     )
 
+    validate_specification = Callable(
+        conda_store_validate_specification,
+        help="callable function taking conda_store and specification as input arguments to apply for validating and modifying a given specification. If there are validation issues with the environment ValueError with message should be raised. If changed you may need to call the default function to preseve many of the trait effects e.g. `c.CondaStore.default_channels` etc",
+        config=True,
+    )
+
     @property
     def session_factory(self):
         if hasattr(self, "_session_factory"):
@@ -196,12 +296,11 @@ class CondaStore(LoggingConfigurable):
         """Ensure that conda-store default namespaces exists"""
         namespace = api.get_namespace(self.db, name=self.default_namespace)
         if namespace is None:
-            self.db.add(orm.Namespace(name=self.default_namespace))
+            api.create_namespace(self.db, name=self.default_namespace)
 
     def ensure_directories(self):
         """Ensure that conda-store filesystem directories exist"""
         os.makedirs(self.store_directory, exist_ok=True)
-        os.makedirs(self.environment_directory, exist_ok=True)
 
     def ensure_conda_channels(self):
         """Ensure that conda-store allowed channels and packages are in database"""
@@ -234,13 +333,16 @@ class CondaStore(LoggingConfigurable):
         # Create Namespace if namespace if it does not exist
         namespace_model = api.get_namespace(self.db, name=namespace)
         if namespace_model is None:
-            namespace = orm.Namespace(name=namespace)
-            self.db.add(namespace)
+            namespace = api.create_namespace(self.db, name=namespace)
             self.db.commit()
         else:
             namespace = namespace_model
 
-        specification_model = schema.CondaSpecification.parse_obj(specification)
+        specification_model = self.validate_specification(
+            conda_store=self,
+            namespace=namespace.name,
+            specification=schema.CondaSpecification.parse_obj(specification),
+        )
         specification_sha256 = utils.datastructure_hash(specification_model.dict())
 
         specification = api.get_specification(self.db, sha256=specification_sha256)
@@ -278,6 +380,8 @@ class CondaStore(LoggingConfigurable):
             environment.current_build = build
             self.db.commit()
 
+        return build.id
+
     def create_build(self, environment_id: int, specification_sha256: str):
         specification = api.get_specification(self.db, specification_sha256)
         build = orm.Build(
@@ -301,7 +405,6 @@ class CondaStore(LoggingConfigurable):
 
         (
             tasks.task_update_storage_metrics.si()
-            | tasks.task_update_conda_channels.si()
             | tasks.task_build_conda_environment.si(build.id)
             | group(*artifact_tasks)
             | tasks.task_update_storage_metrics.si()
