@@ -149,25 +149,37 @@ class RBACAuthorizationBackend(LoggingConfigurable):
                 **entity_bindings,
             }
 
-    def entity_roles(self, arn, entity_bindings, authenticated=False):
-        entity_bindings = self.get_entity_bindings(entity_bindings, authenticated)
-
-        roles = set()
-        for entity_arn, entity_roles in entity_bindings.items():
-            if self.compile_arn_regex(entity_arn).match(arn):
-                roles = roles | set(entity_roles)
-        return roles
-
     def convert_roles_to_permissions(self, roles):
         permissions = set()
         for role in roles:
             permissions = permissions | self.role_mappings[role]
         return permissions
 
-    def authorize(self, entity_bindings, arn, permissions, authenticated=False):
-        entity_bindings = self.get_entity_bindings(entity_bindings, authenticated)
-        roles = self.entity_roles(arn, entity_bindings)
-        return permissions <= self.convert_roles_to_permissions(roles)
+    def get_entity_binding_permissions(self, entity_bindings, authenticated=False):
+        entity_bindings = self.get_entity_bindings(
+            entity_bindings=entity_bindings, authenticated=authenticated
+        )
+        return {
+            entity_arn: self.convert_roles_to_permissions(roles=entity_roles)
+            for entity_arn, entity_roles in entity_bindings.items()
+        }
+
+    def get_entity_permissions(self, entity_bindings, arn, authenticated=False):
+        entity_binding_permissions = self.get_entity_binding_permissions(
+            entity_bindings=entity_bindings, authenticated=authenticated
+        )
+        permissions = set()
+        for entity_arn, entity_permissions in entity_binding_permissions.items():
+            if self.compile_arn_regex(entity_arn).match(arn):
+                permissions = permissions | set(entity_permissions)
+        return permissions
+
+    def authorize(
+        self, entity_bindings, arn, required_permissions, authenticated=False
+    ):
+        return required_permissions <= self.get_entity_permissions(
+            entity_bindings=entity_bindings, arn=arn, authenticated=authenticated
+        )
 
 
 class Authentication(LoggingConfigurable):
